@@ -10,6 +10,9 @@ use structopt::StructOpt;
 pub struct RnpCliOptions {
     pub target: SocketAddr,
 
+    #[structopt(short = "m", long = "mode", default_value = "TCP", help = "Specify protocol to use.")]
+    pub protocol: RnpSupportedProtocol,
+
     #[structopt(
         short = "s",
         long = "src-ip",
@@ -70,6 +73,18 @@ pub struct RnpCliOptions {
     )]
     pub parallel_ping_count: u32,
 
+    #[structopt(long, help = "Specify the server name in the pings, such as QUIC.")]
+    pub server_name: Option<String>,
+
+    #[structopt(long, help = "Enable key logger in TLS for helping packet capture.\nPlease note that it might cause RTT to be larger than the real one, because logging key will also take time.")]
+    pub log_tls_key: bool,
+
+    #[structopt(long = "alpn", default_value = "h3-29", help = "ALPN protocol used in QUIC. Specify \"none\" to disable ALPN.\nIt is usually h3-<ver> for http/3 or hq-<ver> for specific version of QUIC.\nFor latest IDs, please check here: https://www.iana.org/assignments/tls-extensiontype-values/tls-extensiontype-values.xhtml#alpn-protocol-ids")]
+    pub alpn_protocol: String,
+
+    #[structopt(long, help = "Calculate the RTT by checking the time of before and after doing QUIC connect instead of estimated RTT from QUIC. Not recommended, as this might cause the RTT time to be larger than the real one.")]
+    pub use_timer_rtt: bool,
+
     #[structopt(
         short = "q",
         long,
@@ -134,6 +149,7 @@ impl RnpCliOptions {
                 _ => panic!("Source IP and Target IP are not both IPv4 or IPv6!"),
             }
         }
+
         if self.source_port_min.is_none() {
             self.source_port_min = Some(rand::thread_rng().gen_range(10000..30000));
         }
@@ -184,7 +200,7 @@ impl RnpCliOptions {
     pub fn to_rnp_core_config(&self) -> RnpCoreConfig {
         let mut config = RnpCoreConfig {
             worker_config: PingWorkerConfig {
-                protocol: RnpSupportedProtocol::TCP,
+                protocol: self.protocol,
                 target: self.target,
                 source_ip: self.source_ip,
                 ping_interval: Duration::from_millis(self.ping_interval_in_ms.into()),
@@ -192,6 +208,14 @@ impl RnpCliOptions {
                     wait_timeout: Duration::from_millis(self.wait_timeout_in_ms.into()),
                     time_to_live: self.time_to_live,
                     use_fin_in_tcp_ping: self.use_fin_in_tcp_ping,
+                    server_name: self.server_name.as_ref().and_then(|s| Some(s.to_string())),
+                    log_tls_key: self.log_tls_key,
+                    alpn_protocol: if self.alpn_protocol.to_uppercase() != String::from("NONE") {
+                        Some(self.alpn_protocol.clone())
+                    } else {
+                        None
+                    },
+                    use_timer_rtt: self.use_timer_rtt,
                 },
             },
             worker_scheduler_config: PingWorkerSchedulerConfig {
@@ -212,10 +236,7 @@ impl RnpCliOptions {
                 text_log_path: self.text_log_path.clone(),
                 show_result_scatter: self.show_result_scatter,
                 show_latency_scatter: self.show_latency_scatter,
-                latency_buckets: match &self.latency_buckets {
-                    Some(buckets) => Some(buckets.clone()),
-                    None => None,
-                },
+                latency_buckets: self.latency_buckets.as_ref().and_then(|buckets| Some(buckets.clone())),
             },
         };
 
@@ -240,6 +261,7 @@ mod tests {
         assert_eq!(
             RnpCliOptions {
                 target: "10.0.0.1:443".parse().unwrap(),
+                protocol: RnpSupportedProtocol::TCP,
                 source_ip: "0.0.0.0".parse().unwrap(),
                 source_port_min: None,
                 source_port_max: None,
@@ -252,6 +274,10 @@ mod tests {
                 time_to_live: None,
                 use_fin_in_tcp_ping: false,
                 parallel_ping_count: 1,
+                server_name: None,
+                log_tls_key: false,
+                alpn_protocol: String::from("h3-29"),
+                use_timer_rtt: false,
                 no_console_log: false,
                 csv_log_path: None,
                 json_log_path: None,
@@ -269,6 +295,7 @@ mod tests {
         assert_eq!(
             RnpCliOptions {
                 target: "10.0.0.1:443".parse().unwrap(),
+                protocol: RnpSupportedProtocol::TCP,
                 source_ip: "10.0.0.2".parse().unwrap(),
                 source_port_min: None,
                 source_port_max: None,
@@ -281,6 +308,10 @@ mod tests {
                 time_to_live: None,
                 use_fin_in_tcp_ping: false,
                 parallel_ping_count: 10,
+                server_name: None,
+                log_tls_key: false,
+                alpn_protocol: String::from("h3-29"),
+                use_timer_rtt: false,
                 no_console_log: true,
                 csv_log_path: None,
                 json_log_path: None,
@@ -292,6 +323,8 @@ mod tests {
             RnpCliOptions::from_iter(&[
                 "rnp.exe",
                 "10.0.0.1:443",
+                "-m",
+                "tcp",
                 "-s",
                 "10.0.0.2",
                 "-n",
@@ -317,6 +350,7 @@ mod tests {
         assert_eq!(
             RnpCliOptions {
                 target: "10.0.0.1:443".parse().unwrap(),
+                protocol: RnpSupportedProtocol::QUIC,
                 source_ip: "10.0.0.2".parse().unwrap(),
                 source_port_min: Some(1024),
                 source_port_max: Some(2048),
@@ -329,6 +363,10 @@ mod tests {
                 time_to_live: Some(128),
                 use_fin_in_tcp_ping: true,
                 parallel_ping_count: 10,
+                server_name: Some(String::from("localhost")),
+                log_tls_key: true,
+                alpn_protocol: String::from("hq-29"),
+                use_timer_rtt: true,
                 no_console_log: true,
                 csv_log_path: Some(PathBuf::from("log.csv")),
                 json_log_path: Some(PathBuf::from("log.json")),
@@ -340,6 +378,8 @@ mod tests {
             RnpCliOptions::from_iter(&[
                 "rnp.exe",
                 "10.0.0.1:443",
+                "--mode",
+                "quic",
                 "--src-ip",
                 "10.0.0.2",
                 "--src-port-min",
@@ -361,6 +401,12 @@ mod tests {
                 "--use-fin",
                 "--parallel",
                 "10",
+                "--server-name",
+                "localhost",
+                "--log-tls-key",
+                "--alpn",
+                "hq-29",
+                "--use-timer-rtt",
                 "--no-console-log",
                 "--log-csv",
                 "log.csv",
@@ -389,6 +435,10 @@ mod tests {
                         wait_timeout: Duration::from_millis(1000),
                         time_to_live: Some(128),
                         use_fin_in_tcp_ping: false,
+                        server_name: None,
+                        log_tls_key: false,
+                        alpn_protocol: None,
+                        use_timer_rtt: false,
                     },
                 },
                 worker_scheduler_config: PingWorkerSchedulerConfig {
@@ -411,6 +461,7 @@ mod tests {
             },
             RnpCliOptions {
                 target: "10.0.0.1:443".parse().unwrap(),
+                protocol: RnpSupportedProtocol::TCP,
                 ping_count: 4,
                 ping_until_stopped: false,
                 warmup_count: 1,
@@ -423,6 +474,10 @@ mod tests {
                 time_to_live: Some(128),
                 use_fin_in_tcp_ping: false,
                 parallel_ping_count: 1,
+                server_name: None,
+                log_tls_key: false,
+                alpn_protocol: String::from("none"),
+                use_timer_rtt: false,
                 no_console_log: false,
                 csv_log_path: None,
                 json_log_path: None,
@@ -437,7 +492,7 @@ mod tests {
         assert_eq!(
             RnpCoreConfig {
                 worker_config: PingWorkerConfig {
-                    protocol: RnpSupportedProtocol::TCP,
+                    protocol: RnpSupportedProtocol::QUIC,
                     target: "10.0.0.1:443".parse().unwrap(),
                     source_ip: "10.0.0.2".parse().unwrap(),
                     ping_interval: Duration::from_millis(1500),
@@ -445,6 +500,10 @@ mod tests {
                         wait_timeout: Duration::from_millis(2000),
                         time_to_live: Some(128),
                         use_fin_in_tcp_ping: true,
+                        server_name: Some(String::from("localhost")),
+                        log_tls_key: true,
+                        alpn_protocol: Some(String::from("h3")),
+                        use_timer_rtt: true,
                     },
                 },
                 worker_scheduler_config: PingWorkerSchedulerConfig {
@@ -467,6 +526,7 @@ mod tests {
             },
             RnpCliOptions {
                 target: "10.0.0.1:443".parse().unwrap(),
+                protocol: RnpSupportedProtocol::QUIC,
                 ping_count: 4,
                 ping_until_stopped: true,
                 warmup_count: 3,
@@ -479,6 +539,10 @@ mod tests {
                 time_to_live: Some(128),
                 use_fin_in_tcp_ping: true,
                 parallel_ping_count: 1,
+                server_name: Some(String::from("localhost")),
+                log_tls_key: true,
+                alpn_protocol: String::from("h3"),
+                use_timer_rtt: true,
                 no_console_log: true,
                 csv_log_path: Some(PathBuf::from("log.csv")),
                 json_log_path: Some(PathBuf::from("log.json")),
