@@ -110,64 +110,11 @@ impl PingResultProcessor for PingResultProcessorLatencyBucketLogger {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use chrono::{TimeZone, Utc};
-    use std::{io, time::Duration};
-    use crate::ping_clients::ping_client::PingClientError::{PreparationFailed, PingFailed};
+    use crate::rnp_test_common;
 
     #[test]
     fn latency_bucket_logger_should_work() {
-        let ping_results = vec![
-            PingResult::new(
-                &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
-                1,
-                "TCP",
-                "1.2.3.4:443".parse().unwrap(),
-                "5.6.7.8:8080".parse().unwrap(),
-                false,
-                Duration::from_millis(10),
-                false,
-                None,
-            ),
-            PingResult::new(
-                &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
-                1,
-                "TCP",
-                "1.2.3.4:443".parse().unwrap(),
-                "5.6.7.8:8080".parse().unwrap(),
-                false,
-                Duration::from_millis(1000),
-                true,
-                None,
-            ),
-            PingResult::new(
-                &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
-                1,
-                "TCP",
-                "1.2.3.4:443".parse().unwrap(),
-                "5.6.7.8:8080".parse().unwrap(),
-                false,
-                Duration::from_millis(0),
-                false,
-                Some(PingFailed(Box::new(io::Error::new(
-                    io::ErrorKind::ConnectionRefused,
-                    "connect failed",
-                )))),
-            ),
-            PingResult::new(
-                &Utc.ymd(2021, 7, 6).and_hms_milli(9, 10, 11, 12),
-                1,
-                "TCP",
-                "1.2.3.4:443".parse().unwrap(),
-                "5.6.7.8:8080".parse().unwrap(),
-                false,
-                Duration::from_millis(0),
-                false,
-                Some(PreparationFailed(Box::new(io::Error::new(
-                    io::ErrorKind::AddrInUse,
-                    "address in use",
-                )))),
-            ),
-        ];
+        let ping_results = rnp_test_common::generate_ping_result_test_samples();
 
         let mut logger =
             PingResultProcessorLatencyBucketLogger::new(&vec![0.1, 0.5, 1.0, 10.0, 50.0, 100.0]);
