@@ -55,7 +55,17 @@ impl PingResultProcessorLatencyScatterLogger {
 }
 
 impl PingResultProcessor for PingResultProcessorLatencyScatterLogger {
-    fn process(&mut self, ping_result: &PingResult) {
+    fn process_ping_result(&mut self, ping_result: &PingResult) {
+        // Skip warmup pings in analysis.
+        if ping_result.is_warmup() {
+            return;
+        }
+
+        // Skip preparation errors in analysis, since it is not a remote issue.
+        if ping_result.is_preparation_error() {
+            return;
+        }
+
         let (row, col) = self.get_ping_history_item_pos(ping_result.source().port() as u32);
         let bit_mask_bit = 1 << col;
 
@@ -71,7 +81,7 @@ impl PingResultProcessor for PingResultProcessorLatencyScatterLogger {
         }
     }
 
-    fn done(&mut self) {
+    fn rundown(&mut self) {
         println!("\n=== Latency scatter map (in milliseconds) ===\n");
 
         println!(
